import { Application, Utils } from '@nativescript/core';
import { FlutterCommon } from './common';
let flutterEngineGroup;
var FlutterDelegate = /** @class */ (function (_super) {
    __extends(FlutterDelegate, _super);
    function FlutterDelegate() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    FlutterDelegate.prototype.applicationDidFinishLaunchingWithOptions = function (application, launchOptions) {
        configureFlutterEngineGroup();
        return true;
    };
    FlutterDelegate.ObjCProtocols = [UIApplicationDelegate, FlutterAppLifeCycleProvider, FlutterPluginRegistry];
    return FlutterDelegate;
}(FlutterAppDelegate));
export { FlutterDelegate };
export function configureFlutterEngineGroup() {
    flutterEngineGroup = FlutterEngineGroup.alloc().initWithNameProject('ns_flutter_engine', null);
}
export function init() {
    if (global.isIOS) {
        Application.ios.delegate = FlutterDelegate;
    }
}
export class Flutter extends FlutterCommon {
    createNativeView() {
        if (!this.id) {
            throw new Error(`Flutter requires an 'id' property set to match your Dart entry point name.`);
        }
        this.engine = flutterEngineGroup.makeEngineWithEntrypointLibraryURI(this.id, null);
        GeneratedPluginRegistrant.registerWithRegistry(this.engine);
        this.flutterViewController = FlutterViewController.alloc().initWithEngineNibNameBundle(this.engine, null, null);
        return this.flutterViewController.view;
    }
    initNativeView() {
        this.platformChannel = FlutterBasicMessageChannel.messageChannelWithNameBinaryMessengerCodec('nativescript', this.flutterViewController.binaryMessenger, FlutterStringCodec.sharedInstance());
        this.platformChannel.setMessageHandler(this._methodCallHandler.bind(this));
    }
    disposeNativeView() {
        if (this.platformChannel) {
            this.platformChannel.setMessageHandler(null);
            this.platformChannel = null;
        }
    }
    sendMessage(type, data, callback) {
        if (this.platformChannel) {
            const message = {
                type,
            };
            if (data) {
                message.data = data;
            }
            const json = JSON.stringify(message, null, 0);
            if (callback) {
                this.platformChannel.sendMessageReply(json, (result) => {
                    this.notify({
                        eventName: Flutter.sendMessageReplyEvent,
                        object: this,
                        data: result,
                    });
                    if (callback) {
                        callback(result);
                    }
                });
            }
            else {
                this.platformChannel.sendMessage(json);
            }
            if (Flutter.DEBUG) {
                console.log('NativeScript sent message to Flutter:', json);
            }
        }
    }
    _methodCallHandler(message, result) {
        const flutterMessage = JSON.parse(message);
        if (flutterMessage) {
            if (Flutter.DEBUG) {
                console.log('Flutter called NativeScript with type:', flutterMessage.type);
                console.log('data:', flutterMessage.data);
            }
            if (this.channel?.[flutterMessage.type]) {
                this.channel[flutterMessage.type](flutterMessage.data);
            }
        }
    }
    onLoaded() {
        super.onLoaded();
        // we do this on onLoaded as the viewControllers are not properly setup on createNativeView
        // eslint-disable-next-line @typescript-eslint/no-this-alias
        let vcParent = this;
        while (vcParent && !vcParent.viewController) {
            vcParent = vcParent.parent;
        }
        const vc = vcParent?.viewController || Utils.ios.getRootViewController();
        if (vc) {
            vc.addChildViewController(this.flutterViewController);
            this.flutterViewController.didMoveToParentViewController(vc);
        }
    }
}
//# sourceMappingURL=index.ios.js.map